/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
****************************************************************************/

#ifndef SMART_TOOLS_OUTPUT_H
#define SMART_TOOLS_OUTPUT_H

#include <string>
#include <iostream>
#include "stassert.h"
#include "smarttools.h"
#include "ststring.h"
#include <sys/ioctl.h>

/*!
\brief StOutput enables programs to write text with background and foreground colors.

It works just like cout from the C++ std library, but it has added support
for colors. It can write text with given foreground and background colors,
with the option to paint the whole line with the background color.

*/

class StOutput
{
	public:
		StOutput(){ init(); }
		StOutput( SmartTools::COutputType type );
		
		void setBackgroundColor( SmartTools::Color color );
		void setForegroundColor( SmartTools::Color color );
		void setDefaultForegroundColor( SmartTools::Color color );
		void setOutput( SmartTools::COutputType type ){ outputType = type; }
		
		/** Reset colors to default value */
		void clearColors(){     foregroundColor=""; backgroundColor=""; normalColor=""; }
		void clearBackgroundColor(){ backgroundColor=""; }
		void clearForegroundColor(){ foregroundColor=""; }
		void clearNormalColor(){ normalColor=""; }
		
		/** If enabled is false, then no output is printed. get() function works fine. */
		void setEnabled( bool enabled ) { this->enabled = enabled; }
		
		/*!
			Set print start position. Really it puts 'pos' number of spaces
			before output and when output reaches end of line in terminal,
			rest of text is wrapped into next line starting with the same
			number of spaces.
			Zero means normal output.
			
			\warning This works only, when cursor is at the beggining of line.
			
			\param pos - how many spaces should I put before actual output
		*/
		void setStartPosition( unsigned pos ) { tab = pos; }
		
		/*!
			If enabled, words that don't fit in one line will be taken to
			next line in one piece (without cuting). Disabled for default.
		*/
		void setWordWrap( bool wordWrap ) { this->wordWrap = wordWrap; }
		
		/** If the type is Out_Error, use the printError function for output. Otherwise, use printNormal.*/
		void print( const StString& str ) const;
		
		/** It works like print, but returns output in StString. */
		StString get( const StString& str ) const;
		
		/**
		Print str to the standard error stream.
		\note The usual destination of stderr is the text terminal, but you can
		redirect it to a file like this: program 2> file.
		*/
		void printError( const StString& str ) const;
		
		/**
		Print str to the standard output stream.
		\note The usual destination is the text terminal, but you can
		redirect it to a file like this: program > file.
		*/
		void printNormal( const StString& str ) const;
		
		/** Prints the text and puts endl at the end. It can also paint the whole line.*/
		void println( const StString& str, bool paintWholeLine=false );
		
		/**
		Return carriage and enable to clear all lines in standard output or error
		streams depending on output type.
		\param clearLine clear all lines.
		*/
		void carriageReturn( bool clearLine = true );
		
		/** Flush the output buffer. */
		void flush() const {    std::cout << std::flush; }
		
		const StOutput& operator<<( const StString& str ) const { print(str); return *this; }
		StOutput& operator<<( SmartTools::Color color ) { setForegroundColor(color); return *this; }
		
		void rememberLastColor( bool remember ){ _rememberLastColor = remember; }
		
	private:
		bool enabled;
	
		SmartTools::COutputType outputType;
		StString backgroundColor;
		StString foregroundColor;
		StString normalColor;
		SmartTools::Color lastBackgroundColor;
		SmartTools::Color lastNormalColor;
		SmartTools::Color lastForegroundColor;
		winsize window;
		bool _rememberLastColor;
		unsigned tab; // tabbing of the text
		bool wordWrap;
		
		void init();
		
		/*
		It is used to paint or clear all lines.
		\return window size
		*/
		//int getWindowSize();
		
		/*! \return formated StString by setStartPosition() and setWordWrap() criteria. */
		StString reformat( const StString& str ) const;
	
};

static StOutput stout;
static StOutput sterr( SmartTools::Out_Error );

#endif //SMART_TOOLS_OUTPUT_H
