/***************************************************************************
Copyright (c) 2006 Mateusz Bilski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_SERVER_H
#define SMART_TOOLS_SERVER_H

#include "stsocket.h"
#include "stthread.h"
#include "ststring.h"
#include "stclient.h"
#include "stvaluelist.h"

#include <unistd.h>
#include <netinet/in.h>

/*!
	\brief Unix socket server
	
	There are two ways of using StServer. You can set the onConnectPointer, onDisconnectPointer
	and all onDataPointers in StClients. If any event happens, the right function will be called.
	The another way is a loop with a check if there is a new connection, new disconnection or data.
	
	\sa StServer
*/

class StServer : public StSocket, protected StThread
{
	public:
		enum Errors { E_UNKNOWN, E_NOT_SOCKET, E_ACCESS, E_ALREADY_IN_USE, E_BOUND, E_WOULD_BLOCK,
		              E_ABORTED, E_NOT_LISTENING };
		
		/*! 	
			Create an endpoint for communication.
			\param domain specifies a communication domain.
		*/
		StServer( int domain = PF_UNIX ) : StSocket( domain ) { init(); }
		~StServer();
		
		/*!
			Bind a port to a socket and start a thread.
			\note If you want to set the number of max connections, you should do so before executing this function.
			\warning The domain must be AF_INET or PF_INET.
		*/
		bool bind( int port );
		
		/*!
			Bind a filename to a socket and start a thread.
			\warning The domain must be AF_UNIX or PF_UNIX
		*/
		bool bind( const StString& filename );
		
		/*! 
			Set the max number of connections.
			\note This should be executed before the bind function.
		*/
		void setMaxConnections( unsigned num ) { maxConnections = num; }
		
		/*! \return number of connections. */
		int countConnections() { return connected.size(); };
		
		/*! \return true if there is a new connection. */
		bool isNewConnection() { return !newConnected.empty(); };
		
		/*! Return a pointer to the next connected client and remove it from the newConnected list. */
		StClient getNextConnected();
		
		/*! \return true if there is new data. */
		bool isNewData() { return !newData.empty(); };
		
		/*! Return a pointer to the client with new data and remove it from the newData list. */
		StClient getNextWithData();
		
		/*! /return true if there is a new disconnection. */
		bool isNewDisconnection() { return !disconnected.empty(); };
		
		/*! Return a copy of the next disconnected client and remove it from the disconnected list. */
		StClient getNextDisconnected();
		
		/*!
			Set a pointer to a function which will be called when there is a new connection.
			\note If the pointer is not NULL, connected clients will not be added to connected list.
		*/
		void setOnConnect( void (*func)( StClient ) ) { onConnectPointer = func; };
		
		/*!
			Set a pointer to a function which will be called when there is a new disconnection.
			\note If the pointer is not NULL, disconnected clients will not be added to the disconnected list.
		*/
		void setOnDisconnect( void (*func)( StClient ) ) { onDisconnectPointer = func; };
		
		/*! \return a pointer to the list of connected clients. */
		StValueList<StClient> getConnected() { return connected; }
	protected:
		virtual void run();
	private:
		void init() { onConnectPointer = onDisconnectPointer = NULL; maxConnections = 20; }
		StClient findBySocket( int socket );
		
		void (*onConnectPointer)( StClient );
		void (*onDisconnectPointer)( StClient );
		
		unsigned maxConnections;
		
		StValueList<StClient> connected, disconnected, newConnected, newData;
};

#endif //SMART_TOOLS_SERVER_H
